; footswitch.asm   read a footswitch for a guitar effect.  Debounce and toggle.  Switch on/off LED.
;   use power-down sleep.  Requires 4.7mS low on INT0 pin to wake up.  
;  83 millisecond period on Timer used to debounce switch

.nolist
.include "tn11def.inc"
.list

;.device  ATtiny11
;    		Atmel AVR Tiny11
;	RESET/	PIN 1     | VCC  PIN 8  +5V
;		PIN 2 PB3 | PB2  PIN 7 LED ON active low
;   		PIN 3 PB4 | INT0 PIN 6 footswitch
;	        PIN 4 Gnd | PB0  PIN 5 Effect ON =  logic low

    
.equ	EffectONpin	= 0
.equ	LEDpin		= 2 
.equ	SwitchPin	= 1
.equ  	SwitchPort	= Pinb

;*********************************************************************************************
;  Register definitions

.def	temp	= r16
.def	toggler	= r17

.cseg
.org   $0000
	rjmp 	reset  	;$000	Reset handler
.org	INT0addr		;$001	External Interrupt0 Vector Address  
	reti		; awake from power-down sleep and clear the INT0 flag
.org	PCINTaddr	;$002	Pin Change Handler
	reti		
.org	OVF0addr	;$003	Overflow0 Interrupt Vector Address
	rjmp OVT0	
.org	ACIaddr 	;$004	;$004	Analog Comparator Interrupt Vector Address
	reti		


;********************************************************
;
;	Main Code
;
;********************************************************
reset:
; set-up ports for input/output and pull-up resistors
	ldi	temp, (1<< LEDpin) | (1<< EffectONpin)
	out	ddrb, temp
	sbi	portb, EffectONpin	; 4053 switch to activate effect (default OFF)
	cbi	portb, LEDpin		; start with LED off
	sbi	portb, SwitchPin	; turn on pull-up resistor for footswitch   

	clr	temp
	out	TCNT0, temp
	ldi	temp, (1 << TOV0)	; clear interrupt flag
	out	TIMSK, temp
	ldi	temp, ( (1<<CS02) | (0<<CS01) | (0<<CS00) ) ;1MHz/256
	out	TCCR0, temp		; start timer interrupt
	
	ldi	toggler, 0b11111101	; EOR toggle effect switches and LED
	clt	; timer interrupt occurred when set
	
	ser	temp			;clear INT0 and GP interrupt flags(write logic 1 to clear flags)
	out	GIFR, temp		
	
	; set INT0 as low level activated
	ldi	temp, (1<<INT0)		; enable INT0 interrupt
	out	GIMSK, temp		; to catch the footswitch press (to ground) on PB1
	
	sei	; switch on the general interrupt enabler

;*****************************************************************************
;
;  GetSwitch:  Main Loop of the Program  99.9% of time spent here
;
;*****************************************************************************

GetSwitch:	
	ldi	temp, (1<<SE) | (1<<SM) ;  clears ISC01 and ISC00 which selects low level INT0
	out	MCUCR, temp  ; enter power-down sleep mode.  Footswitch press wakes up CPU.
	sleep	
		
; there was a footswitch press.  The INT0 interrupt was only RETI, but the INT0 flag was cleared.
	clt
GS10:	brtc	GS10 ; wait until timer overflow  max period 80 milliseconds
	clt
GS20:	brtc	GS20;  wait another 1/12 second   83milliseconds
	clt
	sbis	SwitchPort, SwitchPin ; debounced switch still pressed?
	rjmp	GetSwitch

GS25:	in	temp, portb   ; toggle effect activation
	eor	temp, toggler
	out	portb, temp
	
	; wait until switch is released
GS30:	sbis	SwitchPort, SwitchPin
	rjmp	GS30
	; debounce switch release
	clt
GS40:	brtc	GS40 ; wait until timer overflow  max period 80 milliseconds
	clt
GS50:	brtc	GS50;  wait another 1/12 second   83 milliseconds
	clt
	sbis	SwitchPort, SwitchPin ; debounced switch still pressed?
	rjmp	GS50	
	
	ser	temp	; clear INTF0 or GP interrupt flags before sleep (write logic 1 to clear flags)
	out	GIFR, temp
	
	in	temp, GIMSK	; re-enable INT0 interrupt
	ori	temp, (1<<INT0)  ; in order to catch the next footswitch press when Tiny11 is sleeping
	out	GIMSK, temp
	
	rjmp	GetSwitch   ; repeat forever


;******************************************************************************
;
;	Timer Overflow Interrupt Code
;
;   This interrupt is executed every 65 milliseconds  @15 times/sec
;
;******************************************************************************
OVT0:	set    ;  T flag 
 	reti	
